#---------------------------------------------------------------------------
# Code to run different estimation methods presented in Strebel et al., Bird Study, 2020
# Nicolas Strebel, January 2020
#---------------------------------------------------------------------------

### Settings and data prep ####
# Load libraries
library("jagsUI") # AND make sure jags is installed on your computer

# Create folder to store BUGS models
dir.create("BUGS_models")

# MCMC settings
ni.tot <- 100; nb <- 50; nt <- 1; nc <- 3 # Test run
ni.tot <- 10000; nb <- 5000; nt <- 10; nc <- 3

# Load data  
# Species: Emberiza cia (Rock bunting)
# Area: a square of 4672 km2, in south-western Switzerland 
load("AppendixS2.RData")
attach(dat)
str(dat)     # Get overview of these data
#List of 4
# $ ydata  :'data.frame':        13976 obs. of  3 variables:
#  ..$ site  : chr [1:13976] "571_145" "572_107" "572_153" "572_155" ...
#  ..$ source: chr [1:13976] "casual" "casual" "casual" "casual" ...
#  ..$ ndet  : num [1:13976] 0 0 0 0 0 0 0 0 0 0 ...
# $ kov    :'data.frame':        4672 obs. of  9 variables:
#  ..$ x           : int [1:4672] 571500 572500 573500 574500 575500 576500 577500 578500 579500 580500 ...
#  ..$ y           : int [1:4672] 164500 164500 164500 164500 164500 164500 164500 164500 164500 164500 ...
#  ..$ z.buildings : num [1:4672] 1.311 -0.183 -0.248 -0.177 -0.37 ...
#  ..$ z.elevation : num [1:4672] -0.701 -0.67 -0.765 -0.743 -0.479 ...
#  ..$ z.farmland  : num [1:4672] 1.5777 -0.0418 0.2716 -0.4075 -0.512 ...
#  ..$ z.forest    : num [1:4672] -0.948 -0.051 -0.672 -0.5 1.192 ...
#  ..$ z.grassland : num [1:4672] 0.43 1.114 -0.013 -0.013 -0.214 ...
#  ..$ z.structures: num [1:4672] -0.509 -0.617 -0.509 -0.509 0.355 ...
# $ kovs   : chr [1:6] "buildings" "elevation" "farmland" "forest" ...
# $ glm.dat:'data.frame':        231 obs. of  2 variables:
#  ..$ site: chr [1:231] "571_106" "571_111" "571_143" "572_148" ...
#  ..$ ndet: num [1:231] 0 0 0 0 0 0 3 0 0 0 ...

# What this all means:
# ---------------------------------------------------------------------------
# ydata: observation data; ndet:detected or not for source=casual, n territories found for source=mapping
# kov: each row represents one grid cell of the study area, contains data on habitat covariates
# kovs: covariates to consider for subsequent analyses
# glm.dat: territory mapping "synthesis", number of found territories per mapped grid cell


### 2) Statistical extrapolation of territory mapping data ####
### 2a) Simple extrapolation of territory mapping results ----

# Add elevation (which is probably the most powerful explanatoy covariate 
# for all things biological at the scale of Switzerland)
for(i in glm.dat$site) glm.dat[glm.dat$site==i,"z.elevation"] <- kov[i,"z.elevation"] 

# Elevation limits based on the data
elev.range <- range(glm.dat$z.elevation[glm.dat$ndet>0])

# Mean density per sampled grid cell times entire number of grid cells in study area
round(mean(glm.dat$ndet) * nrow(kov))

# The same, but only considering grid cells within the (observed) elevational range of the species
mean.dens.within.range <- mean(glm.dat$ndet[glm.dat$z.elevation>=min(elev.range) & glm.dat$z.elevation>=min(elev.range)]) # Mean dens elevation within range of the species
n.gridcells.within.range <- sum(kov$z.elevation>=min(elev.range) & kov$z.elevation<=max(elev.range)) # Number of gridcells within range of species
round(mean.dens.within.range*n.gridcells.within.range) # Result of 2a)


### 2b) Poisson GLM ----
# Model file
cat(paste0("
model {
  ### likelihood
  # Found number of territories 
  for (o in 1:nobs) {
    n[o] ~ dpois(lambda[sitenr[o]])
  }
  
  # Predict
  for (i in 1:nsite) {
    N[i] ~ dpois(lambda[i]) # N = Predicted number of territories
    log(lambda[i]) <- mu + ",  paste0("e.",kovs," * z.",kovs,"[i]",collapse=" + \n      "), "
  }
  
  # Sum of N
  n.tot <- sum(N)
           
  ### Priors
  # Intercept
  mu ~ dnorm(0, 0.01)
  # Priors on Covariates
  ", paste0("e.",kovs," ~ dnorm(0, 0.01)",collapse="\n  ") ,"
}
"), file = paste0("BUGS_models/Poisson_GLM.txt"))

# Get the row-number of the object kov for each row of glm.dat
for (i in glm.dat$site) glm.dat$sitenr[glm.dat$site==i] <- which(rownames(kov)==i)

# Data bundle
bugs.data <- list(
  nsite = nrow(kov),
  n = glm.dat$ndet,
  sitenr = glm.dat$sitenr,
  nobs = nrow(glm.dat))
for(k in grep("z.",colnames(kov),value = T)) bugs.data[[k]] <- kov[,k]

# Parameters to monitor
params <- c("n.tot") # to check convergence, also include "mu" and paste0("e.",kovs)

# Run jags (24 sec on my computer)
glm.out <- jags(data = bugs.data, parameters.to.save = params,model.file = "BUGS_models/Poisson_GLM.txt",
            n.chains = nc, n.thin = nt, n.iter = ni.tot, n.burnin = nb,parallel=T)

# Grab the population size estimate (n.tot)
round(glm.out$summary["n.tot",])  # Result of 2b)


### 2c) Binomial Mixture Model ----
# Model file
cat(paste0("
model {
  ### likelihood
  # Observation model
  logit(p.ind) <- lp.ind
  for (o in 1:nobs) {
    n[o] ~ dbin(p.ind,N[sitenr[o]])
  }
  
  # Predict latent state
  for (i in 1:nsite) {
    N[i] ~ dpois(lambda[i])
    log(lambda[i]) <- mu + ",  paste0("e.",kovs," * z.",kovs,"[i]",collapse=" + \n      "), "
  }
  
  # Sum of N
  n.tot <- sum(N)

  ### Priors
  # Intercept
  mu ~ dnorm(0, 0.01)
  # Priors on Covariates
  ", paste0("e.",kovs," ~ dnorm(0, 0.01)",collapse="\n  ") ,"
  # Det Prob
  lp.ind ~ dnorm(0,0.01)
}
"), file = paste0("BUGS_models/Binomial_Mixture_Model.txt"))

# Create vector indicating the row-number of kov for each row of obs data
binmix.dat <- ydata[ydata$source=="mapping",]
for (i in binmix.dat$site) binmix.dat$sitenr[binmix.dat$site==i] <- which(rownames(kov)==i)

# Data bundle
bugs.data <- list(
  nsite = nrow(kov),
  n = binmix.dat$ndet,
  sitenr = binmix.dat$sitenr,
  nobs = nrow(binmix.dat))
for(k in grep("z.",colnames(kov),value = T)) bugs.data[[k]] <- kov[,k]

# Parameters to monitor
params <- c("n.tot") # to check convergence, also include "mu", "lp.ind" and paste0("e.",kovs)

# Inits
nst <- rep(0,times = nrow(kov))
n.tmp <- tapply(binmix.dat$ndet,binmix.dat$sitenr,max,na.rm=T)
for (i in as.numeric(rownames(n.tmp))) nst[i] <- n.tmp[as.character(i)]

# Run jags (48 sec on my computer)
binmix.out <- jags(data = bugs.data, inits = function(){list(N=nst)}, parameters.to.save = params,model.file = "BUGS_models/Binomial_Mixture_Model.txt",
            n.chains = nc, n.thin = nt, n.iter = ni.tot, n.burnin = nb,parallel=T)

# Grab the population size estimate (n.tot)
round(binmix.out$summary["n.tot",])  # Result of 2c)


### 3) Extrapolation based on range size estimates from site-occupancy models ####
### Fit site-occupancy model to get an estimate of range size  ----
# Model file
cat(paste0("
model {
  ### likelihood
  # Observation model
  logit(p[1]) <- lp[1]
  logit(p[2]) <- lp[2]
  for (o in 1:nobs) {
    y[o] ~ dbern(p[source[o]]*Z[sitenr[o]])  # Detection / nondetection data
  }
  
  # Predict latent state
  for (i in 1:nsite) {
    Z[i] ~ dbern(occ[i])
    logit(occ[i]) <- mu + ",  paste0("e.",kovs," * z.",kovs,"[i]",collapse=" + \n      "), "
  }
  
  # Sum of Occupancy
  occ.tot <- sum(occ)
           
  ### Priors
  # Priors on Covariates
  ", paste0("e.",kovs," ~ dnorm(0, 0.01)",collapse="\n  ") ,"
           
  # Intercept fixed effects
  mu ~ dnorm(0, 0.01)
           
  # Allow for different det prob according to source
  lp[1] ~ dnorm(0,0.01)
  lp[2] ~ dnorm(0,0.01)
}
"), file = paste0("BUGS_models/SiteOcc.txt"))

# Create vector indicating the row-number of kov for each row of obs data
for(i in ydata$site) ydata$sitenr[ydata$site==i] <- which(rownames(kov)==i)

# Data bundle
bugs.data <- list(
  nsite = nrow(kov),
  y = ifelse(ydata$ndet>0,1,0),
  sitenr = ydata$sitenr,
  source = ifelse(ydata$source=="mapping",2,1),
  nobs = nrow(ydata))
for(k in grep("z.",colnames(kov),value = T)) bugs.data[[k]] <- kov[,k]

# Parameters to monitor
params <- c("occ.tot","occ") # to check convergence, also consider "lp", "mu" and paste0("e.",kovs)

# Inits
zst <- rep(0,times = nrow(kov))
z.tmp <- tapply(ifelse(ydata$ndet>0,1,0),ydata$sitenr,max,na.rm=T)
for (i in as.numeric(rownames(z.tmp))) zst[i] <- z.tmp[as.character(i)]

# Run jags (5.2 min on my computer)
siteocc.out <- jags(data = bugs.data, inits = function() list(Z=zst), parameters.to.save = params,model.file = "BUGS_models/SiteOcc.txt",
  n.chains = nc,  n.thin = nt, n.iter = ni.tot, n.burnin = nb,parallel=T)

# Grab the range size estimate (occ.tot)
round(siteocc.out$summary["occ.tot",])


### 3a) Range size estimate * density from literature ----
dens.literature <- 5 # Give a value for max density per grid cell, here I arbitrarily chose 5
dens.literature * siteocc.out$summary["occ.tot","50%"] # Result from 3a)


### 3b) Range size estimate corrected for Home Range ----
home.range.radius.literature <- 0.2 # Give radius of home range relative to grid cell size,
#   here I arbitrarily chose 0.2 (= 200 meters)
siteocc.out$summary["occ.tot","50%"]/(1 + home.range.radius.literature^2 * pi + 4 * home.range.radius.literature)  # Result from 3b)


### 3c) Binomial Mixture estimate corrected for Home Range ----
binmix.out$summary["n.tot","50%"]/(1 + home.range.radius.literature^2 * pi + 4 * home.range.radius.literature) 
# Result from 3c)


### 3d) Range size estimate * density from territory mapping ----
# Extract site occupancy model estimates for sites with territory mapping data
occ.estimates <- siteocc.out$summary[paste0("occ[",glm.dat$sitenr,"]"),"50%"]

# Fit a linear model for '# territories detected' ~ 'site occupancy model estimates'
mod <- lm(glm.dat$ndet ~ occ.estimates + I(occ.estimates^2) - 1) # Quadratic, through 0/0

# converge estimated occupancy per site to density 
dens.estim.site <- siteocc.out$summary[paste0("occ[",1:nrow(kov),"]"),"50%"]*mod$coefficients[1] + siteocc.out$summary[paste0("occ[",1:nrow(kov),"]"),"50%"]^2*mod$coefficients[2]
round(sum(dens.estim.site)) # Result from 3d)


### 4) Extrapolation of regional population size estimates ####
# Load required libraries for spatial analysis
lapply(c("sp","raster","rgeos","rgdal"),require, character.only = TRUE)

# Create a spatial polygon representing the sub-area for which a population estimate exists
coords <- cbind(x=c(625,625,640,640,625)*1000, y=c(110,135,135,110,110)*1000) # Indicate the coordinates
pol <- Polygons(list(Polygon(coords)), ID=1) # Create a polygon
region <- SpatialPolygons(list(pol)) # Create a spatial polygon
# or load a shape: region<-rgdal::readOGR("folder_name","shape_name")

# Convert kov to raster, cell values = occupancy estimate 
kov.grid <- cbind(kov[,c("x","y")],"occ"=siteocc.out$summary[paste0("occ[",1:nrow(kov),"]"),"50%"])
coordinates(kov.grid) <- ~x+y
gridded(kov.grid) <- T
rtopoly <- rasterToPolygons(raster(kov.grid))

# Intersect sub-area with the entire study area
i <- intersect(rtopoly,region)

# Calculate the sum of the estimates of occupancy for the sub-area only
sum.occ.region <- sum(i@data[,"occ"] * gArea(i,byid=TRUE)/1000000) # Occupancy per grid cell has to be multiplied with the proportion of the corresponding grid cell laying within the sub-area (not always 100%)

# Sum of the estimates of occupancy (range size) over the entire study area
occ.tot <- siteocc.out$summary["occ.tot","50%"]

# Population size estimate for the sub-area only, here I arbitrarily  chose 500
n.reg <- 500 

# Divide population size estimate for the sub-area by occupancy estimate for the sub-area 
#   and multiply with total occupancy estimate to get a population size estimate for the entire study area
round(n.reg/sum.occ.region*occ.tot) # Result from 4)